package com.b510.hbase.util.dao.impl;

import java.io.IOException;
import java.util.List;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.hbase.Cell;
import org.apache.hadoop.hbase.CellUtil;
import org.apache.hadoop.hbase.HBaseConfiguration;
import org.apache.hadoop.hbase.HColumnDescriptor;
import org.apache.hadoop.hbase.HTableDescriptor;
import org.apache.hadoop.hbase.MasterNotRunningException;
import org.apache.hadoop.hbase.TableName;
import org.apache.hadoop.hbase.ZooKeeperConnectionException;
import org.apache.hadoop.hbase.client.Delete;
import org.apache.hadoop.hbase.client.Get;
import org.apache.hadoop.hbase.client.HBaseAdmin;
import org.apache.hadoop.hbase.client.HTableInterface;
import org.apache.hadoop.hbase.client.HTablePool;
import org.apache.hadoop.hbase.client.Put;
import org.apache.hadoop.hbase.client.Result;
import org.apache.hadoop.hbase.client.ResultScanner;
import org.apache.hadoop.hbase.client.Scan;

import com.b510.hbase.util.dao.HbaseDao;

/**
 * @author Hongten
 * @created 7 Nov 2018
 */
@SuppressWarnings("deprecation")
public class HbaseDaoImpl implements HbaseDao {

	private static Configuration conf = null;
	private static HBaseAdmin hAdmin;
	private static HTablePool pool;

	private static int defaultPoolSize = 5;

	public HbaseDaoImpl(int poolSize) {
		conf = HBaseConfiguration.create();
		conf.set("hbase.zookeeper.quorum", "node1:2888,node2:2888,node3:2888");
		try {
			hAdmin = new HBaseAdmin(conf);
			// the default pool size is 5.
			pool = new HTablePool(conf, poolSize <= 0 ? defaultPoolSize : poolSize);
		} catch (MasterNotRunningException e) {
			e.printStackTrace();
		} catch (ZooKeeperConnectionException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@Override
	public HTableInterface getHTableFromPool(String tableName) {
		HTableInterface table = pool.getTable(tableName);
		return table;
	}

	@Override
	public boolean isHTableExist(String tableName) {
		try {
			return hAdmin.tableExists(tableName);
		} catch (IOException e) {
			e.printStackTrace();
		}
		return false;
	}

	@Override
	public void createHTable(String tableName, String[] columnFamilys) {
		if (!isHTableExist(tableName)) {
			HTableDescriptor tableDescriptor = new HTableDescriptor(TableName.valueOf(tableName));
			// The Hbase suggested the number of column family should be less than 3.
			// Normally, there only have 1 column family.
			for (String cfName : columnFamilys) {
				HColumnDescriptor hColumnDescriptor = new HColumnDescriptor(cfName);
				tableDescriptor.addFamily(hColumnDescriptor);
			}
			try {
				hAdmin.createTable(tableDescriptor);
			} catch (IOException e) {
				e.printStackTrace();
			}
			System.out.println("The table [" + tableName + "]  is created.");
		} else {
			System.out.println("The table [" + tableName + "]  is existing already.");
		}

	}

	@Override
	public void addRow(String tableName, String rowKey, String columnFamily, String column, String value) {
		if (isHTableExist(tableName)) {
			HTableInterface table = getHTableFromPool(tableName);
			Put put = new Put(rowKey.getBytes());
			put.add(columnFamily.getBytes(), column.getBytes(), value.getBytes());
			try {
				table.put(put);
			} catch (IOException e) {
				e.printStackTrace();
			}
			System.out.println("Insert into table [" + tableName + "], Rowkey=[" + rowKey + "], Column=[" + columnFamily + ":" + column + "], Vlaue=[" + value + "].");
			closeTable(table);
		} else {
			System.out.println("The table [" + tableName + "] does not exist.");
		}
	}

	@Override
	public void getRow(String tableName, String rowKey) {
		if (isHTableExist(tableName)) {
			HTableInterface table = getHTableFromPool(tableName);
			Get get = new Get(rowKey.getBytes());
			Result result;
			try {
				result = table.get(get);
				String columnName = "";
				String timeStamp = "";
				String columnFamily = "";
				String value = "";
				for (Cell cell : result.rawCells()) {
					timeStamp = String.valueOf(cell.getTimestamp());
					columnFamily = new String(CellUtil.cloneFamily(cell));
					columnName = new String(CellUtil.cloneQualifier(cell));
					value = new String(CellUtil.cloneValue(cell));

					System.out.println("Get from table [" + tableName + "], Rowkey=[" + rowKey + "], Column=[" + columnFamily + ":" + columnName + "], Timestamp=[" + timeStamp + "], Vlaue=[" + value + "].");
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
			closeTable(table);
		} else {
			System.out.println("The table [" + tableName + "] does not exist.");
		}
	}

	@Override
	public void getAllRows(String tableName) {
		if (isHTableExist(tableName)) {
			Scan scan = new Scan();
			scanHTable(tableName, scan);
		} else {
			System.out.println("The table [" + tableName + "] does not exist.");
		}
	}

	private void scanHTable(String tableName, Scan scan) {
		try {
			HTableInterface table = getHTableFromPool(tableName);
			ResultScanner results = table.getScanner(scan);
			for (Result result : results) {
				String rowKey = "";
				String columnName = "";
				String timeStamp = "";
				String columnFamily = "";
				String value = "";
				for (Cell cell : result.rawCells()) {
					rowKey = new String(CellUtil.cloneRow(cell));
					timeStamp = String.valueOf(cell.getTimestamp());
					columnFamily = new String(CellUtil.cloneFamily(cell));
					columnName = new String(CellUtil.cloneQualifier(cell));
					value = new String(CellUtil.cloneValue(cell));

					System.out.println("Get from table [" + tableName + "], Rowkey=[" + rowKey + "], Column=[" + columnFamily + ":" + columnName + "], Timestamp=[" + timeStamp + "], Vlaue=[" + value + "].");
				}
			}
			closeTable(table);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void getRowsByRange(String tableName, String startRowKey, String endRowKey) {
		if (isHTableExist(tableName)) {
			Scan scan = new Scan();
			scan.setStartRow(startRowKey.getBytes());
			// not equals Stop Row Key, it mean the result does not include the stop row record(exclusive).
			// the hbase version is 0.98.9
			scan.setStopRow(endRowKey.getBytes());
			scanHTable(tableName, scan);
		} else {
			System.out.println("The table [" + tableName + "] does not exist.");
		}
	}

	@Override
	public void delRow(String tableName, String rowKey) {
		if (isHTableExist(tableName)) {
			HTableInterface table = getHTableFromPool(tableName);
			deleteRow(table, rowKey);
		} else {
			System.out.println("The table [" + tableName + "] does not exist.");
		}
	}

	private void deleteRow(HTableInterface table, String rowKey) {
		Delete del = new Delete(rowKey.getBytes());
		try {
			table.delete(del);
			System.out.println("Delete from table [" + new String(table.getTableName()) + "], Rowkey=[" + rowKey + "].");
			closeTable(table);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void delRowsByRowKeys(String tableName, List<String> rowKeys) {
		if (rowKeys != null && rowKeys.size() > 0) {
			for (String rowKey : rowKeys) {
				delRow(tableName, rowKey);
			}
		}
	}

	@Override
	public void deleteHTable(String tableName) {
		if (isHTableExist(tableName)) {
			try {
				hAdmin.disableTable(tableName.getBytes());
				hAdmin.deleteTable(tableName.getBytes());
				System.out.println("The table [" + tableName + "] is deleted.");
			} catch (IOException e) {
				e.printStackTrace();
			}
		} else {
			System.out.println("The table [" + tableName + "] does not exist.");
		}

	}

	@Override
	public void closeAutoFlush(HTableInterface table) {
		table.setAutoFlush(false, false);
	}

	@Override
	public void closeTable(HTableInterface table) {
		try {
			table.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void closePoolConnection() {
		try {
			pool.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

}
